<?php
declare(strict_types=1);

use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Exception\TransferException;
use Jiwei\EasyHttpSdk\Application;
use Jiwei\EasyHttpSdk\Exception\ApplicationException;
use Jiwei\EasyHttpSdk\Exception\GuiltyResultException;
use Jiwei\EasyHttpSdk\Exception\SdkException;
use Jiwei\EasyHttpSdk\Exception\TimeOutExcetpion;
use Jiwei\EasyHttpSdk\Exception\UnknowResultException;
use Jiwei\EasyHttpSdk\Http\SdkRequest;
use Jiwei\EasyHttpSdk\Option;
use Jiwei\EasyHttpSdk\Policy\DefaultErrorHandlingPolicy;
use PHPUnit\Framework\TestCase;
use Psr\Log\LoggerInterface;
use Psr\Http\Client\ClientInterface;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use GuzzleHttp\Client as GuzzleClient;

class ApplicationTest extends TestCase
{
    private Option $option;
    private LoggerInterface $logger;
    private ClientInterface $client;

    private RequestInterface $mockRequest;

    protected function setUp(): void
    {
        $this->option = $this->getMockBuilder(Option::class)
            ->disableOriginalConstructor()
            ->getMock();

        $this->logger = $this->createMock(LoggerInterface::class);
        $this->client = $this->createMock(GuzzleClient::class);
        $this->mockRequest = $this->createMock(RequestInterface::class);
    }

    /*
     *
     * when the option for the App ID is missing.
     *
     * */
    public function testConstructorThrowsExceptionWhenOptionAppIdIsMissing() : void
    {
        $this->option->method('getAppId')->willReturn('');

        $this->expectException(\RuntimeException::class);
        $this->expectExceptionMessageMatches('/请完善配置信息中的App Key/');

        new Application($this->option, $this->logger);
    }

    /*
    *
    * when the option for the App Secret is missing.
    *
    * */
    public function testConstructorThrowsExceptionWhenOptionAppSecretIsMissing() : void
    {
        $this->option->method('getAppId')->willReturn('chipsuglpf123dd');
        $this->option->method('getAppSecret')->willReturn('');

        $this->expectException(\RuntimeException::class);
        $this->expectExceptionMessageMatches('/请完善配置信息中的App Secret/');

        new Application($this->option, $this->logger);
    }

    /*
    *
    * when the option for the Customer is missing.
    *
    * */
    public function testConstructorThrowsExceptionWhenOptionCustomerIsMissing() : void
    {
        $this->option->method('getAppId')->willReturn('chipsuglpf123dd');
        $this->option->method('getAppSecret')->willReturn('2cojfyk0vx1an4quie3wglr8h97spmmt');
        $this->option->method('getCustomer')->willReturn('');

        $this->expectException(\RuntimeException::class);
        $this->expectExceptionMessageMatches('/请完善配置信息中的 Customer/');

        new Application($this->option, $this->logger);
    }

    /*
    *
    * when the option for the Baseurl is missing.
    *
    * */
    public function testConstructorThrowsExceptionWhenOptionBaseUrlIsMissing() : void
    {
        $this->option->method('getAppId')->willReturn('chipsuglpf123dd');
        $this->option->method('getAppSecret')->willReturn('2cojfyk0vx1an4quie3wglr8h97spmmt');
        $this->option->method('getCustomer')->willReturn('test');
        $this->option->method('getBaseUrl')->willReturn('');

        $this->expectException(\RuntimeException::class);
        $this->expectExceptionMessageMatches('/请检查配置信息中的Base Url/');

        new Application($this->option, $this->logger);
    }

    /*
    *
    * Analog timeout exception
    *
    * */
    public function testSendRequestHandlesTimeOutExcetpion(): void
    {
        $this->setOption();

        $this->client->expects($this->once())
            ->method('send')
            ->willThrowException(new TransferException());

        $this->expectException(TimeOutExcetpion::class);
        $this->expectExceptionMessage('time out.');

        $application = new Application($this->option, $this->logger);
        $application->setClient($this->client);
        $request = $this->createMock(SdkRequest::class);
        $application->sendRequest($request);
    }

    /*
    *
    * Analog application exception
    *
    * */
    public function testSendRequestHandlesApplicationException(): void
    {
        $this->setOption();

        $mockException = $this->createMock(GuzzleException::class);
        $this->client->expects($this->once())
            ->method('send')
            ->willThrowException($mockException);

        $this->expectException(ApplicationException::class);

        $application = new Application($this->option, $this->logger);
        $application->setClient($this->client);
        $request = $this->createMock(SdkRequest::class);
        $application->sendRequest($request);
    }


    /*
    *
    * Analog successful request
    *
    * */
    public function testSendRequestHandlesSuccessfulRequest(): void
    {
        $this->setOption();

        $successfulResponse = $this->createMock(ResponseInterface::class);
        $this->client->expects($this->once())
            ->method('send')
            ->willReturn($successfulResponse);

        $application = new Application($this->option, $this->logger);
        $application->setClient($this->client);
        $request = $this->createMock(SdkRequest::class);
        $response = $application->sendRequest($request);

        $this->assertInstanceOf(ResponseInterface::class, $response);
    }

    /*
     *
     * Simulate ApplicationException
     *
     * */
    public function testResultDecodeHandlesApplicationException(): void
    {
        $this->setOption();
        $response = $this->createMock(ResponseInterface::class);

        $exceptionMessage = "SDK Upstream Failed";

        // 设置模拟响应对象的行为，使其返回状态码 500
        $response->expects($this->once())
            ->method('getStatusCode')
            ->willReturn(500);

        // 断言应该抛出 ApplicationException 异常
        $this->expectException(ApplicationException::class);
        $this->expectExceptionMessage($exceptionMessage);

        // 创建 Application 对象并调用 resultDecode 方法
        $application = new Application($this->option, $this->logger);
        $request = $this->createMock(SdkRequest::class);
        $application->resultDecode($request, $response);
    }

    /*
     *
     * Simulate AuthException
     *
     * */
    public function testResultDecodeHandlesApplicationAuthException(): void
    {
        $this->setOption();
        $response = $this->createMock(ResponseInterface::class);
        $request = $this->createMock(SdkRequest::class); // 创建 SdkRequest 对象

        $exceptionMessage = "auth Failed";

        // 设置模拟响应对象的行为，使其返回状态码 401
        $response->expects($this->any())
            ->method('getStatusCode')
            ->willReturn(401);

        // 断言应该抛出 ApplicationException 异常
        $this->expectException(ApplicationException::class);
        $this->expectExceptionMessage($exceptionMessage);

        // 创建 Application 对象并调用 resultDecode 方法
        $application = new Application($this->option, $this->logger);
        $application->resultDecode($request, $response);
    }

    /*
     *
     * Simulate GuiltyResultException
     *
     * */
    public function testResultDecodeHandlesGuiltyResultException(): void
    {
        $this->setOption();

        // 设置请求和响应对象的模拟
        $request = $this->createMock(SdkRequest::class);
        $response = $this->createMock(ResponseInterface::class);

        $handlingPolicy = $this->createMock(DefaultErrorHandlingPolicy::class);
        $this->option->expects($this->any())
            ->method('handlingPolicy')
            ->willReturn($handlingPolicy);

        $handlingPolicy->expects($this->any())
            ->method('process')
            ->willReturnCallback(function () use ($response) {
                throw new GuiltyResultException('Guilty Result');
            });

        // 创建 Application 对象并调用 resultDecode 方法
        $application = new Application($this->option, $this->logger);

        $this->expectException(SdkException::class);

        $application->resultDecode($request, $response);
    }

    /*
     *
     * Simulate UnknowResultException
     *
     * */
    public function testResultDecodeHandlesUnknowResultException(): void
    {
        $this->setOption();

        // 设置请求和响应对象的模拟
        $request = $this->createMock(SdkRequest::class);
        $response = $this->createMock(ResponseInterface::class);

        $handlingPolicy = $this->createMock(DefaultErrorHandlingPolicy::class);
        $this->option->expects($this->any())
            ->method('handlingPolicy')
            ->willReturn($handlingPolicy);

        $handlingPolicy->expects($this->any())
            ->method('process')
            ->willReturnCallback(function () use ($response) {
                throw new UnknowResultException('Unknow Result');
            });

        // 创建 Application 对象并调用 resultDecode 方法
        $application = new Application($this->option, $this->logger);

        $this->expectException(ApplicationException::class);

        $application->resultDecode($request, $response);
    }

    /*
     *
     * Simulate ResultDecodeHandlesSuccessful
     *
     * */
    public function testResultDecodeHandlesSuccessfulResponse(): void
    {
        $this->setOption();

        // 设置请求和响应对象的模拟
        $request = $this->createMock(SdkRequest::class);
        $response = $this->createMock(ResponseInterface::class);

        // 模拟处理结果
        $handlingResult = ['code' => 'success']; // 请根据实际情况设置处理结果

        // 模拟 handlingPolicy 方法返回的对象
        $handlingPolicy = $this->createMock(DefaultErrorHandlingPolicy::class);
        $this->option->expects($this->once())
            ->method('handlingPolicy')
            ->willReturn($handlingPolicy);

        // 模拟处理结果
        $handlingPolicy->expects($this->once())
            ->method('process')
            ->willReturn($handlingResult);

        // 设置日志记录器模拟
        $application = new Application($this->option, $this->logger);

        // 调用 resultDecode 方法
        $result = $application->resultDecode($request, $response);

        // 断言结果正确
        $this->assertEquals($handlingResult, $result);
    }

    /*
     *
     * option Base
     *
     * */
    public function setOption(): void
    {
        $this->option->method('getAppId')->willReturn('chipsuglpf123dd');
        $this->option->method('getAppSecret')->willReturn('2cojfyk0vx1an4quie3wglr8h97spmmt');
        $this->option->method('getCustomer')->willReturn('test');
        $this->option->method('getBaseUrl')->willReturn('https://jiweidev.jiweinet.com');
    }
}
