<?php
declare(strict_types=1);

use Jiwei\EasyHttpSdk\Option;
use Jiwei\EasyHttpSdk\Middleware\Auth\JwtMiddleware;
use Jiwei\EasyHttpSdk\Policy\DefaultErrorHandlingPolicy;
use Jiwei\EasyHttpSdk\Policy\HandlingPolicyInterface;
use PHPUnit\Framework\TestCase;

class OptionTest extends TestCase
{
    private Option $option;

    public function setUp(): void
    {
        parent::setUp(); // TODO: Change the autogenerated stub

        $this->option = new class () extends Option {
            /** @var string API鉴权地址  */
            private const AUTH_API_ROUTE = "/api/access/token";

            /** @var int API鉴权过期时间 */
            const AUTH_CACHE_EXPIRES_AT = 2400;

            /** @var array<string, string> API HOST */
            const ENDPOINT_HOSTS = [
                "local" => "127.0.0.1",
                "development" => "dev.test.com",
                "production" => "pro.test.com",
            ];

            public function authorization(): Closure
            {
                return function () {
                    echo sprintf("根据Auth API[%s]获取了TOKEN", self::AUTH_API_ROUTE);
                    return "i am a token";
                };
            }
        };
    }


    /**
     *
     * valid AppId
     *
     */
    public function testSetAppId(): void
    {
        $appId = 'test_app_id';
        $result = $this->option->setAppId($appId);

        $this->assertSame($appId, $this->option->getAppId());
        $this->assertInstanceOf(Option::class, $result);
    }


    /**
     *
     * valid AppSecret
     *
     */
    public function testSetAppSecret(): void
    {
        $appSecret = 'test_app_secret';
        $result = $this->option->setAppSecret($appSecret);

        $this->assertSame($appSecret, $this->option->getAppSecret());
        $this->assertInstanceOf(Option::class, $result);
    }

    /**
     *
     * valid Stage
     *
     */
    public function testSetStageWithValidStage(): void
    {
        $stage = 'development';
        $result = $this->option->setStage($stage);

        $this->assertSame($stage, $this->option->getStage());
        $this->assertInstanceOf(Option::class, $result);
    }

    /**
     *
     * invalid Stage
     *
     */
    public function testSetStageWithInvalidStage(): void
    {
        $this->expectException(\InvalidArgumentException::class);

        $invalidStage = 'invalid_stage';
        $this->option->setStage($invalidStage);
    }

    /**
     *
     * Error handling policy validation
     *
     */
    public function testHandlingPolicy(): void
    {
        $policy = $this->option->handlingPolicy();

        $this->assertInstanceOf(HandlingPolicyInterface::class, $policy);
        $this->assertInstanceOf(DefaultErrorHandlingPolicy::class, $policy);
    }


    /**
     *
     * Verify the authentication time
     *
     */
    public function testGetAuthExpires(): void
    {
        $this->assertSame(2400, $this->option->getAuthExpires());
    }

    /**
     *
     * Verify the timeout time
     *
     */
    public function testGetTimeOut(): void
    {
        $this->assertSame(3.0, $this->option->getTimeOut());
    }

    /**
     *
     * Verify the url time
     *
     */
    public function testGetBaseUrlWithDifferentStages(): void
    {
        $this->assertSame('127.0.0.1', $this->option->setStage('local')->getBaseUrl());
        $this->assertSame('dev.test.com', $this->option->setStage('development')->getBaseUrl());
        $this->assertSame('pro.test.com', $this->option->setStage('production')->getBaseUrl());
    }

    /**
     *
     * Verify Debug
     *
     */
    public function testIsDebug(): void
    {
        $this->assertFalse($this->option->setDebug(false)->isDebug());
        $this->assertTrue($this->option->setDebug(true)->isDebug());
    }


    /**
     *
     * Verify Customer
     *
     */
    public function testGetCustomerAndSetCustomer(): void
    {
        $this->assertSame('test_customer', $this->option->setCustomer('test_customer')->getCustomer());
    }


    /**
     *
     * Verify Middleware
     *
     */
    public function testGetAuthorizationMiddlewareWithoutAuthMiddleware(): void
    {
        $this->assertSame(JwtMiddleware::class, $this->option->getAuthorizationMiddleware());
    }
}
